@@ -1,17 +1,17 @@
 use crate::format::Format;
 use crate::items::tokens::AtomToken;
-use crate::parse::{self, Parse, Parser};
+use crate::parse::{self, Parse, TokenStream};
 use crate::span::Span;
 
 macro_rules! impl_parse {
     ($name:ident, $value:expr) => {
         impl Parse for $name {
-            fn parse(parser: &mut Parser) -> parse::Result<Self> {
-                let token: AtomToken = parser.parse()?;
+            fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
+                let token: AtomToken = ts.parse()?;
                 if token.value() == $value {
                     Ok(Self(token))
                 } else {
-                    Err(parse::Error::unexpected_token(parser, token.into()))
+                    Err(parse::Error::unexpected_token(ts, token.into()))
                 }
             }
         }
