@@ -35,7 +35,7 @@ pub struct BitstringComprehensionExpr {
 
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub struct BitstringSegment {
-    value: Either<LiteralExpr, Parenthesized<Expr>>,
+    value: Either<Either<LiteralExpr, Box<BitstringExpr>>, Parenthesized<Expr>>,
     size: Maybe<BitstringSegmentSize>,
     ty: Maybe<BitstringSegmentType>,
 }
@@ -54,17 +54,21 @@ pub struct BitstringSegmentType {
 
 impl Format for BitstringSegmentType {
     fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        fmt.format_item(&self.slash)?;
+        self.slash.format(fmt)?;
         for (item, delimiter) in self
             .specifiers
             .items
             .iter()
             .zip(self.specifiers.delimiters.iter())
         {
-            fmt.format_item(item)?;
-            fmt.format_item(delimiter)?;
+            item.format(fmt)?;
+            delimiter.format(fmt)?;
         }
-        fmt.format_item(self.specifiers.items.last().expect("unreachable"))?;
+        self.specifiers
+            .items
+            .last()
+            .expect("unreachable")
+            .format(fmt)?;
         Ok(())
     }
 }
@@ -78,34 +82,20 @@ pub struct BitstringSegmentTypeSpecifier {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::items::expressions::NonLeftRecursiveExpr;
-    use crate::parse::parse_text;
-
-    fn format(text: &str) -> String {
-        crate::FormatOptions::<crate::items::styles::Child<Expr>>::new()
-            .max_columns(20)
-            .format_text(text)
-            .expect("parse or format failed")
-    }
 
     #[test]
     fn bitstring_construct_works() {
         let texts = [
             "<<>>",
             "<<1, 2:16, 3>>",
+            "<<<<\"foo\">>/binary>>",
             indoc::indoc! {"
                 <<1,
                   (foo()):4/little-signed-integer-unit:8,
                   C/binary>>"},
         ];
         for text in texts {
-            let x = parse_text(text).unwrap();
-            if let Expr::NonLeftRecursive(NonLeftRecursiveExpr::Bitstring(x)) = &x {
-                assert!(matches!(**x, BitstringExpr::Construct(_)));
-            } else {
-                panic!("{:?}", x);
-            }
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Expr);
         }
     }
 
@@ -128,13 +118,7 @@ mod tests {
                                  false>>"},
         ];
         for text in texts {
-            let x = parse_text(text).unwrap();
-            if let Expr::NonLeftRecursive(NonLeftRecursiveExpr::Bitstring(x)) = &x {
-                assert!(matches!(**x, BitstringExpr::Comprehension(_)));
-            } else {
-                panic!("{:?}", x);
-            }
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Expr);
         }
     }
 }
