@@ -37,31 +37,36 @@ pub struct BinaryOpCallExpr {
 
 impl Format for BinaryOpCallExpr {
     fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        fmt.format_item(&self.left)?;
-        fmt.format_item(&self.op)?;
-        fmt.with_subregion(Default::default(), |fmt| self.format_right(fmt))?;
+        self.left.format(fmt)?;
+        self.op.format(fmt)?;
+        if fmt
+            .subregion()
+            .forbid_multi_line()
+            .forbid_too_long_line()
+            .enter(|fmt| self.format_right(fmt, false))
+            .is_err()
+        {
+            fmt.subregion().enter(|fmt| self.format_right(fmt, true))?;
+        }
         Ok(())
     }
 }
 
 impl BinaryOpCallExpr {
-    fn format_right(&self, fmt: &mut format::Formatter) -> format::Result<()> {
-        let options = format::RegionOptions::new().noretry();
-        let options = if fmt.multiline_mode().is_recommended() {
-            match self.op.get() {
-                BinaryOp::Send(_) | BinaryOp::Match(_) => options
-                    .newline()
-                    .recommend_multiline()
-                    .indent(format::IndentMode::Offset(4)),
-                _ => options
-                    .newline()
-                    .recommend_multiline()
-                    .indent(format::IndentMode::Offset(0)),
-            }
+    fn format_right(&self, fmt: &mut format::Formatter, multi_line: bool) -> format::Result<()> {
+        if !multi_line {
+            self.right.format(fmt)?;
+        } else if matches!(self.op.get(), BinaryOp::Send(_) | BinaryOp::Match(_)) {
+            fmt.subregion().indent_offset(4).enter(|fmt| {
+                fmt.write_newline()?;
+                self.right.format(fmt)
+            })?;
         } else {
-            options.forbid_multiline()
-        };
-        fmt.with_subregion(options, |fmt| fmt.format_item(&self.right))?;
+            fmt.subregion().enter(|fmt| {
+                fmt.write_newline()?;
+                self.right.format(fmt)
+            })?;
+        }
         Ok(())
     }
 }
@@ -101,14 +106,6 @@ pub enum BinaryOp {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::items::expressions::NonLeftRecursiveExpr;
-    use crate::parse::parse_text;
-    fn format(text: &str) -> String {
-        crate::FormatOptions::<crate::items::styles::Child<Expr>>::new()
-            .max_columns(20)
-            .format_text(text)
-            .expect("parse or format failed")
-    }
 
     #[test]
     fn function_call_works() {
@@ -122,12 +119,7 @@ mod tests {
             "foo:bar(baz)",
         ];
         for text in texts {
-            let x = parse_text(text).unwrap();
-            assert!(matches!(
-                x,
-                Expr::NonLeftRecursive(NonLeftRecursiveExpr::FunctionCall(_))
-            ));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Expr);
         }
     }
 
@@ -135,12 +127,7 @@ mod tests {
     fn unary_op_call_works() {
         let texts = ["-1", "bnot Foo(1, +2, 3)"];
         for text in texts {
-            let x = parse_text(text).unwrap();
-            assert!(matches!(
-                x,
-                Expr::NonLeftRecursive(NonLeftRecursiveExpr::UnaryOpCall(_))
-            ));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Expr);
         }
     }
 
@@ -150,16 +137,13 @@ mod tests {
             "1 + 2",
             "1 - 2 * 3",
             indoc::indoc! {"
-                {A, B, C} = {foo,
-                             bar,
-                             baz} =
-                    qux() /
-                    quux() div 2"},
+                {A, B, C} =
+                    {foo, bar, baz} =
+                        qux() /
+                        quux() div 2"},
         ];
         for text in texts {
-            let x = parse_text(text).unwrap();
-            assert!(matches!(x, Expr::BinaryOpCall(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Expr);
         }
     }
 }
