@@ -1,4 +1,4 @@
-use crate::format::Format;
+use crate::format::{self, Format};
 use crate::items::atoms::{
     CallbackAtom, DefineAtom, IncludeAtom, IncludeLibAtom, OpaqueAtom, RecordAtom, SpecAtom,
     TypeAtom,
@@ -73,7 +73,7 @@ pub struct FunSpec {
     dot: DotSymbol,
 }
 
-#[derive(Debug, Clone, Span, Parse, Format)]
+#[derive(Debug, Clone, Span, Parse)]
 pub struct SpecClause {
     params: Parenthesized<Items<Type>>,
     arrow: Space<RightArrowSymbol>,
@@ -81,6 +81,36 @@ pub struct SpecClause {
     constraint: Maybe<Constraint>,
 }
 
+impl Format for SpecClause {
+    fn format(&self, fmt: &mut format::Formatter) -> format::Result<()> {
+        self.params.format(fmt)?;
+        self.arrow.format(fmt)?;
+        if fmt
+            .subregion()
+            .forbid_multi_line()
+            .forbid_too_long_line()
+            .enter(|fmt| self.r#return.format(fmt))
+            .is_err()
+        {
+            fmt.write_newline()?;
+            if fmt
+                .subregion()
+                .forbid_multi_line()
+                .forbid_too_long_line()
+                .indent_offset(1)
+                .enter(|fmt| self.r#return.format(fmt))
+                .is_err()
+            {
+                fmt.subregion()
+                    .indent_offset(1)
+                    .enter(|fmt| self.r#return.format(fmt))?;
+            }
+        }
+        self.constraint.format(fmt)?;
+        Ok(())
+    }
+}
+
 #[derive(Debug, Clone, Span, Parse, Format)]
 pub struct Constraint {
     when: Space<WhenKeyword>,
@@ -206,16 +236,6 @@ impl IncludeDirective {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::items::styles::Child;
-    use crate::parse::parse_text;
-    use crate::FormatOptions;
-
-    fn format(text: &str) -> String {
-        FormatOptions::<Child<Form>>::new()
-            .max_columns(20)
-            .format_text(text)
-            .expect("parse or format failed")
-    }
 
     #[test]
     fn define_directive_works() {
@@ -225,8 +245,7 @@ mod tests {
             "-define(Baz(A, B), A), {B).",
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::Define(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 
@@ -237,8 +256,7 @@ mod tests {
             r#"-include_lib("path/to/hrl")."#,
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::Include(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 
@@ -254,8 +272,7 @@ mod tests {
             "-elif(true).",
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::Attr(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 
@@ -275,8 +292,7 @@ mod tests {
             ),
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::Attr(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 
@@ -291,8 +307,7 @@ mod tests {
             ),
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::RecordDecl(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 
@@ -318,27 +333,29 @@ mod tests {
             ),
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::FunDecl(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 
     #[test]
     fn fun_spec_works() {
         let texts = [
-            concat!(
-                "-spec foo(T1, T2) -> T3;\n", //
-                "         (T4, T5) -> T6."
-            ),
+            indoc::indoc! {"
+                -spec foo(X) -> X;
+                         (Y) -> Y."},
+            indoc::indoc! {"
+                -spec foo(T1, T2) ->
+                          T3;
+                         (T4, T5) ->
+                          T6."},
             "-spec id(X) -> X when X :: tuple().",
-            concat!(
-                "-callback foo(atom()) -> {atom(),\n",
-                "                          atom()}."
-            ),
+            indoc::indoc! {"
+                -callback foo(atom()) ->
+                              {atom(),
+                               atom()}."},
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::FunSpec(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 
@@ -353,8 +370,7 @@ mod tests {
             ),
         ];
         for text in texts {
-            assert!(matches!(parse_text(text).unwrap(), Form::TypeDecl(_)));
-            assert_eq!(format(text), text);
+            crate::assert_format!(text, Form);
         }
     }
 }
