@@ -1,6 +1,6 @@
 use crate::format::{self, Format, Formatter};
 use crate::items::forms::Form;
-use crate::parse::{self, Parse, Parser};
+use crate::parse::{self, Parse, TokenStream};
 use crate::span::{Position, Span};
 
 #[derive(Debug, Clone)]
@@ -20,12 +20,12 @@ impl Span for Module {
 }
 
 impl Parse for Module {
-    fn parse(parser: &mut Parser) -> parse::Result<Self> {
+    fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
         let mut forms = Vec::new();
-        while !parser.is_eof()? {
-            forms.push(parser.parse()?);
+        while !ts.is_eof()? {
+            forms.push(ts.parse()?);
         }
-        let position = parser.next_token_start_position()?;
+        let position = ts.current_whitespace_token()?.end_position();
         Ok(Self {
             forms,
             eof: Eof { position },
@@ -36,10 +36,10 @@ impl Parse for Module {
 impl Format for Module {
     fn format(&self, fmt: &mut Formatter) -> format::Result<()> {
         for form in &self.forms {
-            fmt.with_subregion(format::RegionOptions::new(), |fmt| fmt.format_item(form))?;
-            fmt.needs_newline();
+            form.format(fmt)?;
+            fmt.write_newline()?;
         }
-        fmt.format_item(&self.eof)?; // For comments and empty macros
+        self.eof.format(fmt)?; // For comments and empty macros
         Ok(())
     }
 }
