@@ -1,5 +1,5 @@
 use crate::format::{self, Format, Formatter};
-use crate::parse::{self, Parse, Parser};
+use crate::parse::{self, Parse, TokenStream};
 use crate::span::{Position, Span};
 use erl_tokenize::values::{Keyword, Symbol};
 
@@ -33,8 +33,19 @@ impl Token {
     }
 }
 
+impl Parse for Token {
+    fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
+        if let Some(token) = ts.next().transpose()? {
+            Ok(token)
+        } else {
+            let position = ts.current_position();
+            Err(parse::Error::UnexpectedEof { position })
+        }
+    }
+}
+
 macro_rules! impl_traits {
-    ($name:ident, $variant:ident, $is_primitive:expr) => {
+    ($name:ident, $variant:ident, $should_be_packed:expr) => {
         impl Span for $name {
             fn start_position(&self) -> Position {
                 self.start
@@ -46,10 +57,10 @@ macro_rules! impl_traits {
         }
 
         impl Parse for $name {
-            fn parse(parser: &mut Parser) -> parse::Result<Self> {
-                match parser.parse()? {
+            fn parse(ts: &mut TokenStream) -> parse::Result<Self> {
+                match ts.parse()? {
                     Token::$variant(token) => Ok(token),
-                    token => Err(parse::Error::unexpected_token(parser, token)),
+                    token => Err(parse::Error::unexpected_token(ts, token)),
                 }
             }
         }
@@ -59,8 +70,8 @@ macro_rules! impl_traits {
                 fmt.write_text(self)
             }
 
-            fn is_primitive(&self) -> bool {
-                $is_primitive
+            fn should_be_packed(&self) -> bool {
+                $should_be_packed
             }
         }
 
@@ -254,48 +265,69 @@ impl Span for CommentToken {
     }
 }
 
+#[derive(Debug, Clone)]
+pub struct WhitespaceToken {
+    start: Position,
+    end: Position,
+}
+
+impl WhitespaceToken {
+    pub fn new(start: Position, end: Position) -> Self {
+        Self { start, end }
+    }
+}
+
+impl Span for WhitespaceToken {
+    fn start_position(&self) -> Position {
+        self.start
+    }
+
+    fn end_position(&self) -> Position {
+        self.end
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::parse::parse_text;
 
     #[test]
     fn atom_works() {
-        assert!(matches!(parse_text("foo").unwrap(), Token::Atom(_)));
+        crate::assert_format!("foo", Token);
     }
 
     #[test]
     fn char_works() {
-        assert!(matches!(parse_text("$a").unwrap(), Token::Char(_)));
+        crate::assert_format!("$a", Token);
     }
 
     #[test]
     fn float_works() {
-        assert!(matches!(parse_text("12.3").unwrap(), Token::Float(_)));
+        crate::assert_format!("12.3", Token);
     }
 
     #[test]
     fn integer_works() {
-        assert!(matches!(parse_text("12").unwrap(), Token::Integer(_)));
+        crate::assert_format!("12", Token);
     }
 
     #[test]
     fn keyword_works() {
-        assert!(matches!(parse_text("case").unwrap(), Token::Keyword(_)));
+        crate::assert_format!("case", Token);
     }
 
     #[test]
     fn string_works() {
-        assert!(matches!(parse_text("\"foo\"").unwrap(), Token::String(_)));
+        crate::assert_format!("\"foo\"", Token);
     }
 
     #[test]
     fn symbol_works() {
-        assert!(matches!(parse_text("-").unwrap(), Token::Symbol(_)));
+        crate::assert_format!("-", Token);
     }
 
     #[test]
     fn variable_works() {
-        assert!(matches!(parse_text("Foo").unwrap(), Token::Variable(_)));
+        crate::assert_format!("Foo", Token);
     }
 }
